package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.dao.IndicatorDictionaryMapper;
import com.ims.eval.dao.IndicatorMapper;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.Indicator;
import com.ims.eval.entity.IndicatorDictionary;
import com.ims.eval.entity.dto.request.IndicatorDTO;
import com.ims.eval.entity.dto.request.IndicatorDictionaryDTO;
import com.ims.eval.entity.dto.response.IndicatorResDTO;
import com.ims.eval.service.IIndicatorDictionaryService;
import com.ims.eval.service.IIndicatorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.Serializable;
import java.net.URLDecoder;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-26
 */
@Service
public class IndicatorServiceImpl extends ServiceImpl<IndicatorMapper, Indicator> implements IIndicatorService {

	@Resource
	private IndicatorDictionaryMapper indicatorDictionaryMapper;

	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;

	@Override
	public IPage<Indicator> list(Integer pageNum, Integer pageSize, String id, String indicatorName, String indicatorCode, String binSection, String binStage, String dept, String company) {

		QueryWrapper<Indicator> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<Indicator> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(Indicator::getId, id);
		}

		if (StringUtils.isNotEmpty(indicatorName)) {
			qw.lambda().like(Indicator::getIndicatorName, indicatorName);
		}

		if (StringUtils.isNotEmpty(indicatorCode)) {
			qw.lambda().like(Indicator::getIndicatorCode, indicatorCode);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(Indicator::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(Indicator::getBinStage, binStage);
		}

		if (StringUtils.isNotEmpty(dept)) {
			qw.lambda().eq(Indicator::getDeptId, dept);
		}

		if (StringUtils.isNotEmpty(company)) {
			qw.lambda().eq(Indicator::getCompany, company);
		}

		qw.lambda().orderByAsc(Indicator::getOrderNum);

		IPage<Indicator> list = baseMapper.selectPage(page, qw);

		list.getRecords().stream().forEach(l -> {
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" : CacheContext.ddNameMap.get(l.getBinSection()));
			l.setBinStageName(null == CacheContext.ddNameMap.get(l.getBinStage()) ? "" : CacheContext.ddNameMap.get(l.getBinStage()));
		});

		return list;
	}

	@Override
	public IPage<Indicator> list2(Integer pageNum, Integer pageSize, String id, String indicatorName, String indicatorCode, String binSection, String binStage, String dept, String company, String evaluationCycle) {
		Page<Indicator> page = new Page<>(pageNum, pageSize);
		IPage<Indicator> list = baseMapper.seleclistPage(page, id, indicatorName, indicatorCode, binSection, binStage, dept, company, evaluationCycle);
		List<Indicator> records = list.getRecords();
		records.stream().forEach(r -> {
			r.setDeptName(CacheContext.ddNameMap.get(r.getDeptId()));
		});
		return list;
	}

	@Override
	public List<Indicator> listAll(String id, String indicatorName, String indicatorCode, String binSection, String binStage, String dept, String company, String evaluationCycle) {

		if (evaluationCycle != null) {
			evaluationCycle = evaluationCycle.replaceAll("ALL", "");
		}
		List<Indicator> list = baseMapper.seleclistAll(id, indicatorName, indicatorCode, binSection, binStage, dept, company, evaluationCycle);
		return list;
	}

	@Override
	public Indicator selectByCode(String indicatorCode) {

		Indicator indicator = baseMapper.selectByCode(indicatorCode);
		return indicator;
	}

	@Override
	public List<IndicatorResDTO> getGroupChildCodeByIds(List<String> ids) {
		List<IndicatorResDTO> list = baseMapper.selectGroupChildCode(ids);
		return list;
	}

	@Override
	public List<IndicatorResDTO> listByIds(List<String> indicatorIds) {
		List<IndicatorResDTO> list = baseMapper.selectListByIds(indicatorIds);
		return list;
	}

	@Transactional
	@Override
	public boolean saveGroup(IndicatorDTO dto) throws Exception {
		if (null != dto && (null == dto.getId() || "".equals(dto.getId().trim()))) {
			QueryWrapper<Indicator> qw = new QueryWrapper<>();
			qw.lambda().eq(Indicator::getIndicatorCode, dto.getIndicatorCode());
			qw.lambda().eq(Indicator::getBinSection, dto.getBinSection());
			qw.lambda().eq(Indicator::getBinStage, dto.getBinStage());
			List<Indicator> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("当前指标code已存在");
			}
			dto.setCreateTime(new Date());
		} else {
			dto.setUpdateTime(new Date());
		}

		boolean b = super.saveOrUpdate(dto);
		if (!b) {
			throw new CustomException("指标保存失败");
		}

		for (IndicatorDictionaryDTO dto2 : dto.getDtoList()) {

			if (null == dto2.getChildCode() || "".equals(dto2.getChildCode().trim()) || null == dto2.getChildName() || "".equals(dto2.getChildName().trim())) {
				throw new CustomException("子指标名或编码为空");
			}
			List<IndicatorDictionary> indicatorDictionaryList = new ArrayList<>();
			for (IndicatorDictionary i : dto2.getList()) {

				IndicatorDictionary indicatorDictionary = new IndicatorDictionary();

				String decode = URLDecoder.decode(i.getFormula(), "UTF-8");
				String formula = decode.replaceAll(" ", "+");

				QueryWrapper<IndicatorDictionary> qw = new QueryWrapper<>();
				if (null == i.getId() || "".equals(i.getId().trim())) {
					qw.lambda().eq(IndicatorDictionary::getIndicatorId, dto.getId());
					qw.lambda().eq(IndicatorDictionary::getChildCode, dto2.getChildCode());
					qw.lambda().eq(IndicatorDictionary::getOptionCode, i.getOptionCode());
					qw.lambda().eq(IndicatorDictionary::getFormula, i.getFormula());
					qw.lambda().eq(IndicatorDictionary::getDelFlag, "1");
					List<IndicatorDictionary> list = indicatorDictionaryMapper.selectList(qw);
					if (list.size() > 0) {
						throw new CustomException("当前属性code已存在");
					} else {
						indicatorDictionary.setIndicatorId(dto.getId());
						indicatorDictionary.setChildName(dto2.getChildName());
						indicatorDictionary.setChildCode(dto2.getChildCode());
						indicatorDictionary.setOptionName(null == CacheContext.ddNameMap.get(i.getOptionCode()) ? "未知属性" : CacheContext.ddNameMap.get(i.getOptionCode()));
						indicatorDictionary.setOptionCode(i.getOptionCode());
						indicatorDictionary.setOptionOrder(i.getOptionOrder());
						indicatorDictionary.setCreateTime(new Date());
						indicatorDictionary.setFormula(formula);
						indicatorDictionary.setIsShow(i.getIsShow());
						indicatorDictionary.setIsQuantified(i.getIsQuantified());
						indicatorDictionary.setDelFlag("1");
						indicatorDictionaryList.add(indicatorDictionary);
					}
				} else {
					qw.lambda().eq(IndicatorDictionary::getId, i.getId());
					qw.lambda().eq(IndicatorDictionary::getDelFlag, "1");
					List<IndicatorDictionary> list = indicatorDictionaryMapper.selectList(qw);
					if (list.size() > 0) {
						for (IndicatorDictionary idic : list) {
							if (i.getIndicatorId().equals(idic.getIndicatorId()) && (!dto2.getChildCode().equals(idic.getChildCode()) || !i.getOptionCode().equals(idic.getOptionCode()) || !i.getFormula().equals(idic.getFormula()))) {
								idic.setDelFlag("0");
								indicatorDictionaryService.updateById(idic);
								indicatorDictionary.setIndicatorId(dto.getId());
								indicatorDictionary.setChildName(dto2.getChildName());
								indicatorDictionary.setChildCode(dto2.getChildCode());
								indicatorDictionary.setOptionName(null == CacheContext.ddNameMap.get(i.getOptionCode()) ? "未知属性" : CacheContext.ddNameMap.get(i.getOptionCode()));
								indicatorDictionary.setOptionCode(i.getOptionCode());
								indicatorDictionary.setOptionOrder(i.getOptionOrder());
								indicatorDictionary.setCreateTime(new Date());
								indicatorDictionary.setFormula(formula);
								indicatorDictionary.setIsShow(i.getIsShow());
								indicatorDictionary.setIsQuantified(i.getIsQuantified());
								indicatorDictionary.setDelFlag("1");
								indicatorDictionaryList.add(indicatorDictionary);
							}
						}
					}
				}
			}
			b = indicatorDictionaryService.saveBatch(indicatorDictionaryList);
			if (!b) {
				throw new CustomException("指标明细存失败");
			}
		}
		return b;
	}

	@Override
	public IndicatorDTO getById(String id) {
		//获取指标
		IndicatorDTO indicatorDTO = baseMapper.selectById(id);
		//根据指标获取明细组
		if (null != indicatorDTO) {
			List<IndicatorDictionaryDTO> dtoList = indicatorDictionaryService.list(indicatorDTO.getId(), "", "");
			indicatorDTO.setDtoList(dtoList);
		}
		return indicatorDTO;
	}

	@Override
	public boolean saveOrUpdate(Indicator entity) {

		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			QueryWrapper<Indicator> qw = new QueryWrapper<>();
			qw.lambda().eq(Indicator::getIndicatorCode, entity.getIndicatorCode());
			qw.lambda().eq(Indicator::getBinSection, entity.getBinSection());
			qw.lambda().eq(Indicator::getBinStage, entity.getBinStage());
			List<Indicator> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("当前指标code已存在");
			}

			entity.setCreateTime(new Date());
		} else {
			entity.setUpdateTime(new Date());
		}
		return super.saveOrUpdate(entity);

	}


	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {
		idList.stream().forEach(i -> {
			QueryWrapper<IndicatorDictionary> qw = new QueryWrapper<>();
			qw.lambda().eq(IndicatorDictionary::getIndicatorId, i);
			List<IndicatorDictionary> list = indicatorDictionaryMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				Indicator indicator = baseMapper.selectById(i);
				throw new RuntimeException(indicator.getIndicatorName() + "指标存在关联属性");
			}
		});
		int code = baseMapper.deleteBatchIds(idList);
		if (code > 0) {
			return true;
		}
		return false;
	}
}
